// BackpropNetwork.h
// Dan Pemstein
// Little Celtic Warlock Software 2001
// ANN++ Library 0.1

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef BACKPROP_NETWORK_H
#define BACKPROP_NETWORK_H

#include <stdlib.h>
#include <fstream.h>
#include <iomanip.h>
#include "Network.h"
#include "Neuron.h"
#include "BackpropNeuron.h"
#include "WeightedConnection.h"
#include "SigmoidUnit.h"
#include "InputUnit.h"

class BackpropNetwork: public Network
{
public:
	
	// Constructor
	BackpropNetwork(size_t, size_t *);

	// File load constructor
	BackpropNetwork(const char *);

	virtual ~BackpropNetwork();

	void finalize();

	// Runs Network::run but also places the bias unit into the mix
	void run(double *, double *);
	
	// Training algorithm
	// Size_t is number of training examples.
	void train(const size_t &, double *, const double &);

	// Save a backprop net
	void save(const char*);

	static Neuron *returnInputNeuron();

	static Neuron *returnNeuron();

	static Connection *returnConnection();

private:
	BackpropNeuron * bias_;

};

#endif /* BACKPROP_NETWORK_H */
