// BackpropNeuron.cc
// Dan Pemstein
// Little Celtic Warlock Software 2001
// ANN++ Library 0.1

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef BACKPROP_NEURON_CC
#define BACKPROP_NEURON_CC

#include "BackpropNeuron.h"
#define BPN_ALPHA .1

BackpropNeuron::BackpropNeuron(Connection *con, size_t out, size_t in,
                               size_t excite, Unit *unit)
	:	Neuron(con, out, in, excite, unit),
		error_ (0)
{
}

BackpropNeuron::BackpropNeuron(Unit *unit)
	:	Neuron(0, 0, 0, 0, unit),
		error_ (0)
{
}

BackpropNeuron::~BackpropNeuron()
{
}

void
BackpropNeuron::setError(const double &e)
{
	error_ = e;
}

double
BackpropNeuron::getError()
{
	return error_;
}

void
BackpropNeuron::calcError(const double &rate, const int &n)
{
	double o = unit_->getOutput(); // This neuron's (unweighted) output
	double sum = 0;  // the sum of errors to output nodes
	BackpropNeuron *temp_node;
	WeightedConnection *current = 
		static_cast<WeightedConnection*> (output_);
	while (current != 0) {
		temp_node = static_cast<BackpropNeuron*> (current->value());
		sum += current->weight() * temp_node->getError();
		// Here we update our weights since we've finished with this
		// connection.  (It would be more readable if we had a second
		// method called updateWeights but would waste cycles).
		double delta = (rate * temp_node->getError() * o) +
										(BPN_ALPHA * current->delta_last());
		current->weight(current->weight() + delta);
		current->delta_last(delta);
		current = static_cast<WeightedConnection *> (current->next());
	}
	error_ = o * (1 - o) * sum;
}

#endif /* BACKPROP_NEURON_CC */
