// BackpropNeuron.h
// Dan Pemstein
// Little Celtic Warlock Software 2001
// ANN++ Library 0.1

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef BACKPROP_NEURON_H
#define BACKPROP_NEURON_H

#include "Neuron.h"
#include "WeightedConnection.h"

class BackpropNeuron: public Neuron
{
// A neuron implementation for use with the backprop network class.
// Gives neurons the ability to hold and manipulate error information
// for backpropagation through the network.
public:
	// Constructor
	BackpropNeuron (Connection *, size_t, size_t, size_t, Unit *);

	// In most cases we create Neurons that are unconnected and then
	// use the connect functions in the Network classes to link
	// them.  this constructor makes this convienent.
	BackpropNeuron(Unit *);

	// Destructor
	virtual ~BackpropNeuron();

	// Set this unit's error (used for output units)
	void setError(const double &);

	// Get ths unit's error
	double getError();

	// Calcuate this unit's error.  When this function is called
	// it assumes that all Neurons it outputs to have already had
	// their errors calculated or set. It also updates the weights
	// as it goes along
	void calcError(const double &, const int &);

private:
	double error_;
};

#endif /* BACKPROP_NEURON_H */
