// Connection.h
// Dan Pemstein
// Little Celtic Warlock Software 2001
// ANN++ Library 0.1

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef CONNECTION_H
#define CONNECTION_H

#include <stdlib.h>
#include "Neuron.h"

class Neuron;  // forward declaration

class Connection
{
// A base class for connections between Neurons.  Really just
// a linked list that holds Neurons as its data value.  In
// most cases, users will use the supplied WeightedConnection
// when building nets although more extravagant connections
// can be built with ease.  Also, some nets may not require
// weights on connections.  For example, connection between
// hidden and context nodes are often static in recurrent nets.
public:
	// Default Constructor with no value or next
	Connection();

	// Create a connection with the specified value(s).
	Connection(Neuron *, Connection *next = 0);

	// Destructor
	~Connection();

	// Get the pointer to the next Connection.
	Connection *next () const;

	// Set the pointer to the next Connection
	void next (Connection *);

	// Get (a pointer to) the Neuron contained by the Connection
	Neuron *value ();

	// Set the pointer to the Neuron contained by this Connection
	void value (Neuron *);

	// Set the position (inlayer, innumber, outlayer, outnumber)
	void position (size_t, size_t, size_t,
	               size_t);

	// Return a pointer to an array of size_t showing position
	void position (size_t *);

	// Used by neurons to propogate charges
	virtual void placeCharge(const double &);

	// Used by Networks to see charges produced by neurons
	double getCharge();

private:
	// Neuron held to by the Connection
	Neuron *value_;

	// Pointer to the next Connection
	Connection *next_;

	// The value of the last charge applied to the connection
	// This is used by networks during training.
	double charge_;

	// The position indexes
	size_t inLayer_;
	size_t inNode_;
	size_t outLayer_;
	size_t outNode_;
};

#endif /* CONNECTION_H */
