// Layer.cc
// Dan Pemstein
// Little Celtic Warlock Software 2001
// ANN++ Library 0.1

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef LAYER_CC
#define LAYER_CC

#include "Layer.h"

Layer::Layer ()
	:	max_nodes_ (0),
		num_nodes_(0)
{
}

Layer::Layer (size_t n)
	:	max_nodes_ (n),
		num_nodes_ (0)
{
	// Initialize all the indexes in the map to zero
	for (size_t i = 0; i < max_nodes_; i++)
		nodes_[i] = 0;
}

Layer::~Layer ()
{
	// Clears out neurons in layer.  Neurons take care
	// of their connections
	Neuron *temp;
	for (size_t i = 0; i < max_nodes_; i++) {
		temp = nodes_[i];
		if (temp != 0) {
			nodes_[i] = 0;
			delete temp;
		}
	}
}

void
Layer::resize (const size_t &size)
{
	max_nodes_ = size;
	num_nodes_ = 0;
	for (size_t i = 0; i < max_nodes_; i++)
		nodes_[i] = 0;
}

void
Layer::empty()
{
	num_nodes_ = 0;
	Neuron *temp;
	for (size_t i = 0; i < max_nodes_; i++) {
		temp = nodes_[i];
		if (temp != 0) {
			nodes_[i] = 0;
			delete temp;
		}
	}
}

int 
Layer::addNeuron(Neuron *node, const size_t &index)
{
	if (index < max_nodes_) {
		if (nodes_[index] == 0) {
			nodes_[index] = node;
			num_nodes_++;
			return 0;
		}
	}
	return -1;
}
	
int 
Layer::removeNeuron(const size_t &index)
{
	Neuron *temp = nodes_[index];
	if (temp != 0) {
		nodes_[index] = 0;
		delete temp;
		num_nodes_--;
		return 0;
	}
	return -1;
}

int
Layer::get (size_t i, Neuron *&node) 
{
	if (i < max_nodes_) {
		if (nodes_.count(i)) {
			node = nodes_[i];
			return 0;
		}
	}
	return -1;
}

size_t
Layer::size ()
{
	return num_nodes_;
}

int
Layer::isFull()
{
	if (num_nodes_ == max_nodes_)
		return 1;
	else
		return 0;
}

int
Layer::isEmpty()
{
	if (num_nodes_ == 0)
		return 1;
	else
		return 0;
}

/***************** LayerIterator Imlementation **************/

LayerIterator::LayerIterator(const Layer *l)
{
	current_ = 0;
	ilayer_ = l;
}

LayerIterator::~LayerIterator()
{
	current_ = 0;
	ilayer_ = 0;
}

void
LayerIterator::iterateOver(const Layer *l)
{
	current_ = 0;
	ilayer_ = l;
}

void
LayerIterator::first ()
{
	current_ = 0;
}

int
LayerIterator::next()
{
	while (! is_done()) {
		current_++;
		if (ilayer_->nodes_.count(current_) != 0)
			break;
	}
	if (! is_done())
		return 0;
	return -1;
}

int
LayerIterator::is_done () const
{
	if (current_ >= ilayer_->max_nodes_)
		return 1;
	return 0;
}

int
LayerIterator::current_item (size_t &item) const
{
	if (! is_done()) {
		item = current_;
		return 0;
	}
	return -1;
}

#endif /* LAYER_CC */
