// Layer.h
// Dan Pemstein
// Little Celtic Warlock Software 2001
// ANN++ Library 0.1

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef LAYER_H
#define LAYER_H

#include <new>
#include <map>
#include "Neuron.h"

class LayerIterator;

class Layer
{
// A container class representing one layer of nodes
// Can be extended for specific types of layers such as
// input, output, hidden or context.
public:
	// Default Constructor
	Layer ();

	// Standard Constructor
	Layer (size_t);

	// Destructor
	~Layer();

	// Resize this Layer (do the work of the standard constructor
	// when an array situation forced the default to be used)
	void resize(const size_t &);
	
	void empty();
	
	// Add a neuron
	int addNeuron(Neuron *, const size_t &);

	// Remove neuron.  Not a specific one, just a neuron
	int removeNeuron(const size_t &);

	// Access a Neuron at a particular index
	int get(size_t, Neuron *&);

	// Get the size of this layer
	size_t size();

	// See if this layer is full
	int isFull();

	int isEmpty();

	friend class LayerIterator;

private:
	// Map (like a hash) of neurons in the layer
	map<size_t, Neuron *> nodes_;
	
	// Max number of nodes
	size_t max_nodes_;

	size_t num_nodes_;
};

class LayerIterator
{
public:
	//Constructor
	LayerIterator(const Layer *);

	// Destructor
	~LayerIterator();

	void iterateOver(const Layer *);

	void first();

	int next();
	
	int is_done() const;

	// We want the index, not the actual item in this case so that is
	// what we return
	int current_item (size_t &) const;

private:
	const Layer *ilayer_;

	size_t current_;
};

#endif /* LAYER_H */
