// Neuron.cc
// Dan Pemstein
// Little Celtic Warlock Software 2001
// ANN++ Library 0.1

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef NEURON_CC
#define NEURON_CC

#include "Neuron.h"

Neuron::Neuron(Connection *con, size_t out, size_t in, size_t excite,
               Unit *unit)
	:	output_ (con),
		num_outputs_ (out),
		num_inputs_ (in),
		excite_level_ (excite),
		unit_ (unit)
{
}

Neuron::~Neuron()
{
	Connection *temp_head;
	for (int i = 0; i < num_outputs_; i++) {
		temp_head = output_->next();
		delete output_;
		output_ = temp_head;
	}
	// Not necesarry but makes me feel good
	output_ = 0;
	num_outputs_ = num_inputs_ = excite_level_ = 0;
	delete unit_; // the unit attached to this node must be cleaned up
}

int
Neuron::addInput()
{
	num_inputs_++;
	return 0;  // error provided for overriding
}

int 
Neuron::addOutput(Connection *connection)
{
	Connection *temp = output_;
	if (temp == 0) {
		output_ = connection;
	} else {
		while (temp->next() != 0) {
			temp = temp->next();
		}
		temp->next(connection);
		num_outputs_++;
	}
	return 0;
}

int
Neuron::removeInput()
{
	num_inputs_--;
	return 0;  // as with add this can be used by overriding methods
}

int
Neuron::removeOutput(Neuron &node)
{
	Connection *current = output_;
	Connection *prev = 0;
	while (current != 0) {
		if (&node == (current->value())) { // if we've found our node
			if (prev == 0)   // if it is the head
				output_ = current->next();
			else
				prev->next(current->next());
			delete current;
			return 0;
		}
		prev = current;
		current = current->next();
	}
	return -1;
}

void
Neuron::placeCharge(const double &charge)
{
	if (excite_level_ == 0) // We do this at begin because we want to be
		unit_->reset();				// able to look at the unit for learning
	unit_->addCharge(charge);
	excite_level_++;
	if (excite_level_ == num_inputs_) { // reached output threshold
		double output = unit_->getOutput();
		Connection *current = output_;
		while (current != 0) { // tell the outputs
			current->placeCharge(output);
			current = current->next();
		}
		excite_level_ = 0;
	}
}

/************ NeuronIterator Implementation **************/

NeuronIterator::NeuronIterator(const Neuron *n)
{
	current_ = n->output_;
	ineuron_ = n;
}

NeuronIterator::NeuronIterator()
	:	current_ (0),
		ineuron_ (0)
{
}

NeuronIterator::~NeuronIterator()
{
	current_ = 0;
	ineuron_ = 0;
}

void
NeuronIterator::iterateOver(const Neuron *n)
{
	ineuron_ = n;
	current_ = ineuron_->output_;
}

void
NeuronIterator::first ()
{
	current_ = ineuron_->output_;
}

int
NeuronIterator::next ()
{
	if (! is_done()) {
		current_ = current_->next();
		return 0;
	}
	return -1;
}

int
NeuronIterator::is_done () const
{
	if (current_ == 0)
		return 1;
	return 0;
}

int
NeuronIterator::current_item (Connection *&item) const
{
	if (! is_done()) {
		item = current_;
		return 0;
	} else
		return -1;
}

#endif /* NEURON_CC	*/
