// Unit.h
// Dan Pemstein
// Little Celtic Warlock Software 2001
// ANN++ Library 0.1

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef UNIT_H
#define UNIT_H

class Unit
{
// An "abstract" unit class.  Units are encapsulated inside Neurons
// and do the actual calculations within the Neuron.  Standard
// units are Sigmoid and Hyperbolic tangent
public:
	// Reset this unit for the next set of inputs.
	virtual void reset() = 0;

	// Units take in inputs from all the neurons that input into the
	// neuron it resides in, one at a time.  This is that call.
	virtual void addCharge(const double &) = 0;

	// Once a neuron has given the unit all input, this can be used
	// to get an output
	virtual double getOutput() = 0;
	
	virtual ~Unit();
};

#endif /* UNIT_H */
