// WeightedConnection.cc
// Dan Pemstein
// Little Celtic Warlock Software 2001
// ANN++ Library 0.1

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef WEIGHTED_CONNECTION_CC
#define WEIGHTED_CONNECTION_CC

#include "WeightedConnection.h"

WeightedConnection::WeightedConnection()
	:	Connection(),
		weight_ (0),
		delta_last_ (0)
{
}

WeightedConnection::WeightedConnection(Neuron *value, 
                                       Connection *next = 0,
	                                     double weight = 0)
	:	Connection (value, next),
		weight_ (weight),
		delta_last_ (0)
{
}
	
const double 
&WeightedConnection::weight () const
{
	return weight_;
}

void 
WeightedConnection::weight(double weight)
{
	weight_ = weight;
}

void
WeightedConnection::placeCharge(const double &charge)
{
	double total = charge * weight_;
	Connection::placeCharge(total);
}

const double
&WeightedConnection::delta_last () const
{
	return delta_last_;
}

void
WeightedConnection::delta_last(double dl)
{
	delta_last_ = dl;
}

#endif /* WEIGHTED_CONNECTION_CC */
