// WeightedConnection.h
// Dan Pemstein
// Little Celtic Warlock Software 2001
// ANN++ Library 0.1

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef WEIGHTED_CONNECTION_H
#define WEIGHTED_CONNECTION_H

#include "Connection.h"

class WeightedConnection: public Connection
{
// An extension to the Connection class which allows networks to
// weight the connections between neurons.  In practical terms,
// this class is the base connection class in the ANN++ library,
// but Connection is provided for flexibility.
public:
	// Default Constructor
	WeightedConnection();

	// Create a weighted connection with the specified values(s).
	WeightedConnection(Neuron *, Connection *next = 0,
	                   double weight = 0);
	
	// Get (a const reference to) the weight of the connection
	const double &weight () const;

	// Set the weight of the connection
	void weight(double);

	const double &delta_last () const;

	void delta_last(double);

	virtual void placeCharge(const double &);

private:
	double weight_;
	double delta_last_;
};	

#endif /* WEIGHTED_CONNECTION_H */
