#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "BackpropNetwork.h"

int
main (int argc, char *argv[])
{
	// Set the random seed
	srand48(time(NULL));
	
	// Check args and open file
	int nhl;  // the number of hidden layers
	FILE *infile;
	if (argc < 7) {
		cerr << "*** ann++\n"
			<< "*** Args:  in-fn out-fn iterations test_freq "
			<< "rate num_hidden_layers\n"
			<< "*** hidden_layer_size... (1 hidden_layer_size per layer)\n"
			<< "***\n"
			<< "*** NEED MORE ARGUMENTS!\n";
		return 1;
	}
	sscanf(argv[6], "%d", &nhl);
	if (argc != 7 + nhl) {
		cerr << "*** ann++\n"
			<< "*** Args:  in-fn out-fn iterations test_freq "
			<< "rate num_hidden_layers\n"
			<< "*** hidden_layer_size... (1 hidden_layer_size per layer)\n"
			<< "***\n"
			<< "*** NUMBER OF ARGUMENTS DOESN'T MATCH HIDDEN LAYERS!\n";
		return 1;
	}
	if ((infile = fopen(argv[1], "r")) == NULL) {
		cerr << "*** ann++\n"
			<< "*** Args:  in-fn out-fn iterations test_freq "
			<< "rate num_hidden_layers"
			<< "*** hidden_layer_size... (1 hidden_layer_size per layer)\n"
			<< "***\n"
			<< "*** ERROR OPENING FILE " << argv[1] << "!\n";
		return 1;
	}

	// Read the header info
	int bool_in, bool_out, real_in, real_out, train_ex, val_ex, test_ex;
	int in, out, check_ex;
	
	fscanf(infile, "bool_in=%d real_in=%d bool_out=%d real_out=%d\
			training_examples=%d validation_examples=%d test_examples=%d",
			&bool_in, &real_in, &bool_out, &real_out, &train_ex, &val_ex,
			&test_ex);
	
	// We don't care if they are bool or real.  We also train on the
	// train set and test on the val+test sets cause we aren't worried
	// about stopping criteria.
	
	in = bool_in + real_in;
	out = bool_out + real_out;
	check_ex = val_ex + test_ex;

	// Some convienent vars for array sizes
	int tex_size = train_ex * (in + out);
	int cex_size = check_ex * (in + out);

	// Create our train and test arrays.
	double tex[tex_size];
	double cex[cex_size];

	// Fill our arrays
	double *pnt = tex;
	for (int i = 0; i < train_ex; i++) {
		for (int j = 0; j < (in + out); j++) {
			fscanf(infile, "%le", pnt);
			pnt++;
		}
	}
	pnt = cex;
	for (int i = 0; i < check_ex; i++) {
		for (int j = 0; j < (in + out); j++) {
			fscanf(infile, "%le", pnt);
			pnt++;
		}
	}

	// We're done with the file now
	fclose(infile);

	// Create our layer representation from the arguments
	cout << "Getting backpropagation network attributes...\n";
	size_t per[nhl + 2];
	per[0] = (size_t) in;
	cout << "\tInput: " << per[0] << "\n\tHidden: (";
	for (int i = 1; i < nhl + 1; i++) {
		sscanf(argv[6 + i], "%d", &per[i]);
		if (i == nhl) {
			cout << per[i] << ")\n";
		} else {
			cout << per[i] << ", ";
		}
	}
	per[nhl + 1] = (size_t) out;
	cout << "\tOutput: " << per[nhl + 1] << "\n\n";
	

	// Create our network object; fill, connect, and finalize
	cout << "Building backpropagation network...\n";
	BackpropNetwork *net = new BackpropNetwork(nhl + 2, per);
	net->fill(BackpropNetwork::returnInputNeuron,
			      BackpropNetwork::returnNeuron);
	net->connect(BackpropNetwork::returnConnection);
	net->finalize();

	// Read in some variables from the args
	int iterations;
	double rate;
	int test_freq;
	sscanf(argv[3], "%d", &iterations);
	sscanf(argv[5], "%le", &rate);
	sscanf(argv[4], "%d", &test_freq);
	cout << "Testing frequency is " << test_freq << endl;
	cout << "Training rate is " << rate << endl << endl;
	cout << "Training network for " << iterations << " iterations...\n";

	// Initialize our out arrays for run procedures
	double trout[out];
	double chout[out];

	// Create our filename buffer for network saves
	char outfn[256];

	// Ask the user if they want to gnuplot
	cout << "Do you wish to create plots using gnuplot (y/N)?\n";
	char answer = getchar();
	bool gnuplot = 0;
	FILE *plot_data;
	if (answer == 'Y' || answer == 'y') {
		plot_data = fopen("/tmp/ann++_plot.data", "w");
		gnuplot = 1;
	}
	
	// Start training
	for (int i = 1; i <= iterations; i++) {
		net->train(train_ex, tex, rate);
		// Evaluate our performance every 100 epochs
		if (i % test_freq == 0) {
			// Save the network
			sprintf(outfn, "%s_%d.ann\0", argv[2], i);
			net->save(outfn);
			cout << "***********************************************"
				<< "*********************************\n"
				<< "EPOCH " << i << endl;
			// Calculate and print diagnostics on train set
			double train_error = 0.0;
			int right = 0;;
			double right01 = 0.0;
			double right02 = 0.0;
			double right03 = 0.0;
			double right04 = 0.0;
			for (int j = 0; j < train_ex; j++) {
				net->run(&(tex[j * (in + out)]), trout);
				double mhigh = -1;;
				int mhighnum = -1;
				double ihigh = -1;
				int ihighnum = -1;
				for (int k = 0; k < out; k++) {
					train_error += pow(fabs(tex[(j * (in + out)) + k + in] -
							                trout[k]), 2);
					if (trout[k] > mhigh) {
						mhigh = trout[k];
						mhighnum = k;
					}
					if (tex[(j * (in + out)) + k + in] > ihigh) {
						ihigh = tex[(j * (in + out)) + k + in];
						ihighnum = k;
					}
					if (fabs(tex[(j * (in + out)) + k + in] - trout[k]) < .1) {
						right01++;
					}
					if (fabs(tex[(j * (in + out)) + k + in] - trout[k]) < .2) {
						right02++;
					}
					if (fabs(tex[(j * (in + out)) + k + in] - trout[k]) < .3) {
						right03++;
					}
					if (fabs(tex[(j * (in + out)) + k + in] - trout[k]) < .4) {
						right04++;
					}
				}
				if (mhighnum == ihighnum) {
					right++;
				}
			}
			train_error /= train_ex;
			right01 /= train_ex;
			right02 /= train_ex;
			right03 /= train_ex;
			right04 /= train_ex;
			cout << "Training Set:\n"
				<< "Avg Sum of Squared Errors:                              "
				<< train_error << endl;
			if (bool_out > 0 && real_out == 0) {
			cout << "Num Examples Correct(BOOL OUTS):                        "
				<< right << " / " << train_ex << endl;
			}
			cout << "Average Number of Outputs n of Correct Value: (n = .1): "
				   << right01 << " / " << out << endl
					 << "                                              (n = .2): "
				   << right02 << " / " << out << endl
					 << "                                              (n = .3): "
				   << right03 << " / " << out << endl
					 << "                                              (n = .4): "
				   << right04 << " / " << out << endl;
			cout << endl;
			if (gnuplot) { // save date for our plot
				double cor = (double) right / check_ex;
				fprintf(plot_data, "%le %le ", train_error, cor);
			}

			// Calculate and print diags for test set
			double check_error = 0.0;
			right = 0;
			right01 = 0.0;
			right02 = 0.0;
			right03 = 0.0;
			right04 = 0.0;
			for (int j = 0; j < check_ex; j++) {
				net->run(&(cex[j * (in + out)]), chout);
				double mhigh = -1;;
				int mhighnum = -1;
				double ihigh = -1;
				int ihighnum = -1;
				for (int k = 0; k < out; k++) {
					check_error += pow(fabs(cex[(j * (in + out)) + k + in] -
							                chout[k]), 2);
					if (chout[k] > mhigh) {
						mhigh = chout[k];
						mhighnum = k;
					}
					if (cex[(j * (in + out)) + k + in] > ihigh) {
						ihigh = cex[(j * (in + out)) + k + in];
						ihighnum = k;
					}
					if (fabs(cex[(j * (in + out)) + k + in] - chout[k]) < .1) {
						right01++;
					}
					if (fabs(cex[(j * (in + out)) + k + in] - chout[k]) < .2) {
						right02++;
					}
					if (fabs(cex[(j * (in + out)) + k + in] - chout[k]) < .3) {
						right03++;
					}
					if (fabs(cex[(j * (in + out)) + k + in] - chout[k]) < .4) {
						right04++;
					}
				}
				if (mhighnum == ihighnum) {
					right++;
				}
			}
			check_error /= check_ex;
			right01 /= check_ex;
			right02 /= check_ex;
			right03 /= check_ex;
			right04 /= check_ex;
			cout << endl << endl;
			cout << "Validation Set:\n"
				<< "Avg Sum of Squared Errors:                              "
				<< check_error << endl;
			if (bool_out > 0 && real_out == 0) {
			cout << "Num Examples Correct(BOOL OUTS):                        "
				<< right << " / " << check_ex << endl;
			}
			cout << "Average Number of Outputs n of Correct Value: (n = .1): "
				   << right01 << " / " << out << endl
					 << "                                              (n = .2): "
				   << right02 << " / " << out << endl
					 << "                                              (n = .3): "
				   << right03 << " / " << out << endl
					 << "                                              (n = .4): "
				   << right04 << " / " << out << endl;
			cout << endl;
			if (gnuplot) { // save date for our plot
				double cor = (double) right / check_ex;
				fprintf(plot_data, "%le %le\n", check_error, cor);
			}
		} else if (gnuplot) {
				fprintf(plot_data, "? ? ? ?\n");
		}
	}
	if (gnuplot) { // run gnuplot
		fclose(plot_data);
		// Create a temp script
		FILE *plot_script = fopen("/tmp/ann++_script.gp", "w");
		sprintf(outfn, "%s_er.eps", argv[2]);
		fprintf(plot_script, "set terminal postscript eps enhance color 16");
		fprintf(plot_script, "\nset out \'%s\'\n", outfn);
		fprintf(plot_script, "set xlabel \'Epochs\'\n");
		fprintf(plot_script, "set ylabel \'{/Symbol e}^{2}\'\n");
		fprintf(plot_script, "plot \"/tmp/ann++_plot.data\" using 0:1 ");
		fprintf(plot_script,"title \'Train Set\', \"/tmp/ann++_plot.data\"");
		fprintf(plot_script, " using 0:3 title \'Test Set \'\n");
		// correct
		sprintf(outfn, "%s_cor.eps", argv[2]);
		fprintf(plot_script, "\nset out \'%s\'\n", outfn);
		fprintf(plot_script, "set ylabel \'Percent Correct\'\n");
		fprintf(plot_script, "plot \"/tmp/ann++_plot.data\" using 0:2 ");
		fprintf(plot_script,"title \'Train Set\', \"/tmp/ann++_plot.data\"");
		fprintf(plot_script, " using 0:4 title \'Test Set \'");
		fclose(plot_script);
		system("gnuplot /tmp/ann++_script.gp");
		system("rm /tmp/ann++_plot.data");
		system("rm /tmp/ann++_script.gp");
		cout << "Plots saved as " << argv[2] << "_e.eps and "
			<< argv[2] << "_ac.eps\n"; 
	}
}
